


struct ethernet_t {
	bit<48> dstAddr
	bit<48> srcAddr
	bit<16> etherType
}

struct ipv4_t {
	bit<4> version
	bit<4> ihl
	bit<8> diffserv
	bit<16> totalLen
	bit<16> identification
	bit<3> flags
	bit<13> fragOffset
	bit<8> ttl
	bit<8> protocol
	bit<16> hdrChecksum
	bit<32> srcAddr
	bit<32> dstAddr
	bit<80> newfield
}

struct tcp_t {
	bit<16> srcPort
	bit<16> dstPort
	bit<32> seqNo
	bit<32> ackNo
	bit<4> dataOffset
	bit<3> res
	bit<3> ecn
	bit<6> ctrl
	bit<16> window
	bit<16> checksum
	bit<16> urgentPtr
}

struct metadata {
	bit<32> psa_ingress_parser_input_metadata_ingress_port
	bit<32> psa_ingress_parser_input_metadata_packet_path
	bit<32> psa_egress_parser_input_metadata_egress_port
	bit<32> psa_egress_parser_input_metadata_packet_path
	bit<32> psa_ingress_input_metadata_ingress_port
	bit<32> psa_ingress_input_metadata_packet_path
	bit<64> psa_ingress_input_metadata_ingress_timestamp
	bit<8> psa_ingress_input_metadata_parser_error
	bit<8> psa_ingress_output_metadata_class_of_service
	bit<8> psa_ingress_output_metadata_clone
	bit<16> psa_ingress_output_metadata_clone_session_id
	bit<8> psa_ingress_output_metadata_drop
	bit<8> psa_ingress_output_metadata_resubmit
	bit<32> psa_ingress_output_metadata_multicast_group
	bit<32> psa_ingress_output_metadata_egress_port
	bit<8> psa_egress_input_metadata_class_of_service
	bit<32> psa_egress_input_metadata_egress_port
	bit<32> psa_egress_input_metadata_packet_path
	bit<16> psa_egress_input_metadata_instance
	bit<64> psa_egress_input_metadata_egress_timestamp
	bit<8> psa_egress_input_metadata_parser_error
	bit<32> psa_egress_deparser_input_metadata_egress_port
	bit<8> psa_egress_output_metadata_clone
	bit<16> psa_egress_output_metadata_clone_session_id
	bit<8> psa_egress_output_metadata_drop
	bit<16> local_metadata_data
	bit<8> Ingress_tmp
	bit<8> Ingress_tmp_0
	bit<8> Ingress_tmp_1
	bit<8> Ingress_tmp_2
	bit<8> Ingress_key_0
	bit<48> Ingress_tbl_ethernet_dstAddr
	bit<48> Ingress_tbl_ethernet_srcAddr
	bit<16> tmpMask
	bit<8> tmpMask_0
}
metadata instanceof metadata

header ethernet instanceof ethernet_t
header ipv4 instanceof ipv4_t
header tcp instanceof tcp_t

struct psa_ingress_output_metadata_t {
	bit<8> class_of_service
	bit<8> clone
	bit<16> clone_session_id
	bit<8> drop
	bit<8> resubmit
	bit<32> multicast_group
	bit<32> egress_port
}

struct psa_egress_output_metadata_t {
	bit<8> clone
	bit<16> clone_session_id
	bit<8> drop
}

struct psa_egress_deparser_input_metadata_t {
	bit<32> egress_port
}

action NoAction args none {
	return
}

action execute args none {
	mov m.local_metadata_data 0x1
	return
}

table tbl {
	key {
		m.Ingress_key_0 optional
		m.Ingress_tbl_ethernet_dstAddr exact
		m.Ingress_tbl_ethernet_srcAddr exact
	}
	actions {
		NoAction
		execute
	}
	default_action NoAction args none 
	size 0x10000
}


apply {
	rx m.psa_ingress_input_metadata_ingress_port
	mov m.psa_ingress_output_metadata_drop 0x0
	extract h.ethernet
	mov m.tmpMask h.ethernet.etherType
	and m.tmpMask 0xf00
	jmpeq INGRESSPARSERIMPL_PARSE_IPV4 m.tmpMask 0x800
	jmpeq INGRESSPARSERIMPL_PARSE_TCP h.ethernet.etherType 0xd00
	jmp INGRESSPARSERIMPL_ACCEPT
	INGRESSPARSERIMPL_PARSE_IPV4 :	extract h.ipv4
	mov m.tmpMask_0 h.ipv4.protocol
	and m.tmpMask_0 0xfc
	jmpeq INGRESSPARSERIMPL_PARSE_TCP m.tmpMask_0 0x4
	jmp INGRESSPARSERIMPL_ACCEPT
	INGRESSPARSERIMPL_PARSE_TCP :	extract h.tcp
	INGRESSPARSERIMPL_ACCEPT :	mov m.Ingress_tmp 1
	jmpv LABEL_0END h.ethernet
	mov m.Ingress_tmp 0
	LABEL_0END :	mov m.Ingress_tmp_0 1
	jmpv LABEL_1END h.ipv4
	mov m.Ingress_tmp_0 0
	LABEL_1END :	mov m.Ingress_tmp_1 m.Ingress_tmp
	jmpeq LABEL_2FALSE m.Ingress_tmp_1 0x0
	jmpeq LABEL_2FALSE m.Ingress_tmp_0 0x0
	mov m.Ingress_tmp_1 0x1
	jmp LABEL_2END
	LABEL_2FALSE :	mov m.Ingress_tmp_1 0x0
	LABEL_2END :	mov m.Ingress_tmp_2 1
	jmpv LABEL_3END h.tcp
	mov m.Ingress_tmp_2 0
	LABEL_3END :	mov m.Ingress_key_0 m.Ingress_tmp_1
	jmpeq LABEL_4TRUE m.Ingress_key_0 0x1
	jmpeq LABEL_4TRUE m.Ingress_tmp_2 0x1
	mov m.Ingress_key_0 0x0
	jmp LABEL_4END
	LABEL_4TRUE :	mov m.Ingress_key_0 0x1
	LABEL_4END :	mov m.Ingress_tbl_ethernet_dstAddr h.ethernet.dstAddr
	mov m.Ingress_tbl_ethernet_srcAddr h.ethernet.srcAddr
	table tbl
	jmpneq LABEL_DROP m.psa_ingress_output_metadata_drop 0x0
	emit h.ethernet
	emit h.ipv4
	emit h.tcp
	emit h.ethernet
	emit h.ipv4
	emit h.tcp
	tx m.psa_ingress_output_metadata_egress_port
	LABEL_DROP :	drop
}


